#!/usr/bin/perl
###############################################################################
#    Webmin Sysstats - network-lib.pl
#
#    Webmin Sysstats Module
#    Copyright (C) 2001 by David Bouius
#    Bug reports to: dbouius@users.sourceforge.net
#    $Id: network-lib.pl,v 1.7 2001/12/07 10:53:47 gerbier Exp $
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
###############################################################################

sub install_module {
  local ($minfo, $defaults) = ($_[0], $_[1]);

  local (%my_config);

  $my_config{'sample_rate'} = '1';
  $my_config{'db_uid'} = '1';

  &write_config($minfo, \%my_config, $defaults);
  &install_script($minfo, 'network.sh');

  return 1;
}

sub remove_module {
}

sub mod_check_runstop {
  local ($minfo, $my_config, $defaults, $opt) = ($_[0], $_[1], $_[2], $_[3]);

  return $my_config->{'runstop'};
}

sub mod_runstop {
  local ($minfo, $my_config, $defaults, $opt) = ($_[0], $_[1], $_[2], $_[3]);

  $my_config->{'runstop'} = $my_config->{'runstop'} ? '1' : '0';
  &write_config($minfo, $my_config, $defaults);
}

sub mod_check_upgrade {
  return 0;
}

sub mod_upgrade {
}

sub graph_data {
  local ($minfo, $my_config, $defaults, $opt) = ($_[0], $_[1], $_[2], $_[3]);

  $rrdtool = $config{'rrdtool'};
  $sizex = $config{'graph_sizex'};
  $sizey = $config{'graph_sizey'};

  local ($net_num, $sub_display, $index_text)
    = split(/,/, $my_config->{$in{'opt'}});

  local ($interface,
	 $idisplay_mode, $idisplay_active, $idisplay_name,
	 $i_colour, $o_colour,
	 $edisplay_mode, $edisplay_active, $edisplay_name,
	 $ie_colour, $oe_colour, $coll_colour,
	 $runstop
	)
    = split(/,/, $my_config->{'net' . $net_num});

  if ($sub_display eq 1) {
    $rrd_detals  = " DEF:input=" . $minfo->{'dbtables'} . $interface . ".rrd:input:MAX";
    $rrd_detals .= " DEF:output=" . $minfo->{'dbtables'} . $interface . ".rrd:output:MAX";
    $rrd_detals .= " COMMENT:'Trafic     Max       Min  Average\\n'";
    $rrd_detals .= " AREA:input#" . $i_colour . ":'In '";
    $rrd_detals .= " GPRINT:input:MAX:'%6.2lf'";
    $rrd_detals .= " GPRINT:input:MIN:'%6.2lf'";
    $rrd_detals .= " GPRINT:input:AVERAGE:'%6.2lf\\n'";
    $rrd_detals .= " LINE1:output#" . $o_colour . ":'Out'";
    $rrd_detals .= " GPRINT:output:MAX:'%6.2lf'";
    $rrd_detals .= " GPRINT:output:MIN:'%6.2lf'";
    $rrd_detals .= " GPRINT:output:AVERAGE:'%6.2lf\\n'";
    $rrd_detals .= " CDEF:total=input,output,+";
    $rrd_detals .= " GPRINT:total:MAX:'Sum    %6.2lf'";
    $rrd_detals .= " GPRINT:total:MIN:'%6.2lf'";
    $rrd_detals .= " GPRINT:total:AVERAGE:'%6.2lf\\n'";
  } else {
    $rrd_detals  = " -v 'packets'";
    $rrd_detals .= " DEF:ierror=" . $minfo->{'dbtables'} . $interface . ".rrd:ierror:MAX";
    $rrd_detals .= " DEF:oerror=" . $minfo->{'dbtables'} . $interface . ".rrd:ierror:MAX";
    $rrd_detals .= " DEF:collision=" . $minfo->{'dbtables'} . $interface . ".rrd:collision:MAX";
    $rrd_detals .= " COMMENT:'Direction   Max     Min  Average\\n'";
    $rrd_detals .= " LINE1:ierror#" . $ie_colour . ":'In   '";
    $rrd_detals .= " GPRINT:ierror:MAX:'%6.2lf'";
    $rrd_detals .= " GPRINT:ierror:MIN:'%6.2lf'";
    $rrd_detals .= " GPRINT:ierror:AVERAGE:'%6.2lf\\n'";
    $rrd_detals .= " LINE1:oerror#" . $oe_colour . ":'Out  '";
    $rrd_detals .= " GPRINT:oerror:MAX:'%6.2lf'";
    $rrd_detals .= " GPRINT:oerror:MIN:'%6.2lf'";
    $rrd_detals .= " GPRINT:oerror:AVERAGE:'%6.2lf\\n'";
    $rrd_detals .= " LINE1:oerror#" . $coll_colour . ":'Coll.'";
    $rrd_detals .= " GPRINT:collision:MAX:'%6.2lf'";
    $rrd_detals .= " GPRINT:collision:MIN:'%6.2lf'";
    $rrd_detals .= " GPRINT:collision:AVERAGE:'%6.2lf\\n'";
    $rrd_detals .= " CDEF:total=ierror,oerror,collision,+,+";
    $rrd_detals .= " GPRINT:total:MAX:'Sum      %6.2lf'";
    $rrd_detals .= " GPRINT:total:MIN:'%6.2lf'";
    $rrd_detals .= " GPRINT:total:AVERAGE:'%6.2lf\\n'";
  }


  foreach $graph ('hour', 'day', 'week', 'month') {
    if ($graph eq 'hour') {
      $time = "-3600";
      $title = "'The last hour'";
    } elsif ($graph eq 'day') {
      $time = "-86400";
      $title = "'The last day'";
    } elsif ($graph eq 'week') {
      $time = "-604800";
      $title = "'The last week'";
    } elsif ($graph eq 'month') {
      $time = "-18748800";
      $title = "'The last month'";
    }

    $common  = $rrdtool . " graph " . $minfo->{'tmpgname'} . "_" . $graph . ".png";
    $common .= " --start " . $time . " --end -60";
    $common .= " -h " . $sizey . " -w " . $sizex;
    $common .= " -a PNG --title=" . $title;

    $sig = " COMMENT:'\\n' COMMENT:\"Created: `date`\"";

    system($common . $rrd_detals . $sig);
  }
}

sub mod_config {
  local ($minfo, $my_config, $defaults) = ($_[0], $_[1], $_[2]);

  if ($in{'opt'}) {
    if ($in{'opt'} eq 'up') {
      my $tmp = $my_config->{'net' . $in{'idx_mod'}};
      $my_config->{'net' . $in{'idx_mod'}} = $my_config->{'net' . ($in{'idx_mod'} - 1)};
      $my_config->{'net' . ($in{'idx_mod'} - 1)} = $tmp;
    } elsif ($in{'opt'} eq 'down') {
      my $tmp = $my_config->{'net' . $in{'idx_mod'}};
      $my_config->{'net' . $in{'idx_mod'}} = $my_config->{'net' . ($in{'idx_mod'} + 1)};
      $my_config->{'net' . ($in{'idx_mod'} + 1)} = $tmp;
    } elsif ($in{'opt'} eq 'runstop') {
      local ($interface,
	     $idisplay_mode, $idisplay_active, $idisplay_name,
	     $i_colour, $o_colour,
	     $edisplay_mode, $edisplay_active, $edisplay_name,
	     $ie_colour, $oe_colour, $coll_colour,
	     $runstop
	    )
	= split(/,/, $my_config->{'net' . $in{'idx_mod'}});

      $runstop = $runstop ? 0 : 1;
      $my_config->{'net' . $in{'idx_mod'}}
	= join(',', $interface,
	       $idisplay_mode, $idisplay_active, $idisplay_name,
	       $i_colour, $o_colour,
	       $edisplay_mode, $edisplay_active, $edisplay_name,
	       $ie_colour, $oe_colour, $coll_colour,
	       $runstop
	      )
      } elsif ($in{'opt'} eq 'display1') {
	local ($interface,
	       $idisplay_mode, $idisplay_active, $idisplay_name,
	       $i_colour, $o_colour,
	       $edisplay_mode, $edisplay_active, $edisplay_name,
	       $ie_colour, $oe_colour, $coll_colour,
	       $runstop
	      )
	  = split(/,/, $my_config->{'net' . $in{'idx_mod'}});

	$idisplay_active = $idisplay_active ? 0 : 1;
	$my_config->{'net' . $in{'idx_mod'}}
	  = join(',', $interface,
		 $idisplay_mode, $idisplay_active, $idisplay_name,
		 $i_colour, $o_colour,
		 $edisplay_mode, $edisplay_active, $edisplay_name,
		 $ie_colour, $oe_colour, $coll_colour,
		 $runstop
		)
	} elsif ($in{'opt'} eq 'display2') {
	  local ($interface,
		 $idisplay_mode, $idisplay_active, $idisplay_name,
		 $i_colour, $o_colour,
		 $edisplay_mode, $edisplay_active, $edisplay_name,
		 $ie_colour, $oe_colour, $coll_colour,
		 $runstop
		)
	    = split(/,/, $my_config->{'net' . $in{'idx_mod'}});

	  $edisplay_active = $edisplay_active ? 0 : 1;
	  $my_config->{'net' . $in{'idx_mod'}}
	    = join(',', $interface,
		   $idisplay_mode, $idisplay_active, $idisplay_name,
		   $i_colour, $o_colour,
		   $edisplay_mode, $edisplay_active, $edisplay_name,
		   $ie_colour, $oe_colour, $coll_colour,
		   $runstop
		  )
	  } elsif ($in{'opt'} eq 'rm') {
	    local ($interface,
		   $idisplay_mode, $idisplay_active, $idisplay_name,
		   $i_colour, $o_colour,
		   $edisplay_mode, $edisplay_active, $edisplay_name,
		   $ie_colour, $oe_colour, $coll_colour,
		   $runstop
		  )
	      = split(/,/, $my_config->{'net' . $in{'idx_mod'}});

	    while ($my_config->{'net' . $in{'idx_mod'}} ne '') {
	      $my_config->{'net' . $in{'idx_mod'}++} = $my_config->{'net' . ($in{'idx_mod'})};
	    }

	    $my_config->{'net' . ($in{'idx_mod'} - 1)} = '';

	    &write_config($minfo, $my_config, $defaults);

	    unlink ($minfo->{'dbtables'} . $interface . '.rrd');
	  }
    &write_config($minfo, $my_config, $defaults);
  }
  print "<table border width='100%'>\n";
  print  "<tr $cb>";
  print   "<td>";
  print    "<table width='100%'>\n";
  print     "<tr><td><b> $text{'module_subtitle'}</b></td></tr>\n";

  print     "<tr><td><table border width='100%'>\n";
  print     "<td $tb><b>$text{'module_subtitle_name'}</b></td>";
  print     "<td $tb><b>$text{'module_display_name'}</b></td>";
  print     "<td $tb><b>$text{'module_subtitle_disp_colour'}</b></td>";
  print     "<td $tb width='" . (22 * 6) . "'><b>$text{'module_subtitle_controls'}</b></td>";

  my $idx_mod = 1;
  while ($my_config->{'net' . $idx_mod} ne '') {
    local ($interface,
	   $idisplay_mode, $idisplay_active, $idisplay_name,
	   $i_colour, $o_colour,
	   $edisplay_mode, $edisplay_active, $edisplay_name,
	   $ie_colour, $oe_colour, $coll_colour,
	   $runstop
	  )
      = split(/,/, $my_config->{'net' . $idx_mod});

    print    "<tr $cb>";
    print     "<td><b><a href='edit_module.cgi?idx=$in{'idx'}&opt=edit&idx_mod=$idx_mod'>$interface</a></b></td>";
    print     "<td></td><td></td><td align='right'>";
    if ($access{'configure'}) {
      if ($idx_mod > 1) {
	print "<A href='edit_module.cgi?idx=$in{'idx'}&opt=up&idx_mod=$idx_mod'>";
	print "<IMG src='images/up.png' border='0'>";
	print "</a>";
      } else {
	print "<IMG src='images/space.png' border='0'>";
	print "<IMG src='images/space.png' border='0'>";
      }
      print "<IMG src='images/space.png' border='0'>\n";
      if ($my_config->{'net' . ($idx_mod + 1)} ne '') {
	print "<A href='edit_module.cgi?idx=$in{'idx'}&opt=down&idx_mod=$idx_mod'>";
	print "<IMG src='images/down.png' border='0'>";
	print "</a>";
      } else {
	print "<IMG src='images/space.png' border='0'>";
	print "<IMG src='images/space.png' border='0'>";
      }
      print "<IMG src='images/space.png' border='0'>\n";
      if ($runstop) {
	print "<A href='edit_module.cgi?idx=$in{'idx'}&opt=runstop&idx_mod=$idx_mod'>";
	print "<IMG src='images/record.png' border='0'>";
	print "</a>";
      } else {
	print "<A href='edit_module.cgi?idx=$in{'idx'}&opt=runstop&idx_mod=$idx_mod'>";
	print "<IMG src='images/pause.png' border='0'>";
	print "</a>";
      }
      print "<IMG src='images/space.png' border='0'>\n";
      print "<IMG src='images/space.png' border='0'>\n";
      print "<IMG src='images/space.png' border='0'>\n";
      print "<IMG src='images/space.png' border='0'>\n";
      print "<A href='edit_module.cgi?idx=$in{'idx'}&opt=rm&idx_mod=$idx_mod'>";
      print "<IMG src='images/trash.png' border='0'>";
      print "</a>";
    }
    print    "</td></tr>\n<tr>";
    print    "<td></td>";
    print     "<td>$idisplay_name</td>";
    print     "<td><table width='100%'><tr>";
    print      "<td width='30%'><font color='$i_colour'><b>";
    print      $text{'colour_' . $i_colour} ? $text{'colour_' . $i_colour} : "<pre>#$i_colour </pre>";
    print      "</b></font></pre></td>";
    print      "<td width='30%'><font color='$o_colour'><b>";
    print      $text{'colour_' . $o_colour} ? $text{'colour_' . $o_colour} : "<pre>#$o_colour </pre>";
    print      "</b></font></pre></td>";
    print     "<td width='30%'></td></tr></table></td><td align='right'>";
    if ($access{'configure'}) {
      if (!$idisplay_active) {
	print "<A href='edit_module.cgi?idx=$in{'idx'}&opt=display1&idx_mod=$idx_mod'>";
	print "<IMG src='images/nodata.png' border='0'>";
	print "</a>";
      } else {
	print "<A href='edit_module.cgi?idx=$in{'idx'}&opt=display1&idx_mod=$idx_mod'>";
	print "<IMG src='images/showdata.png' border='0'>";
	print "</a>";
      }
      print "<IMG src='images/space.png' border='0'>\n";
      print "<IMG src='images/space.png' border='0'>\n";
      print "<IMG src='images/space.png' border='0'>\n";
    }
    print "</td></tr>\n<tr>";
    print    "<td></td>";
    print     "<td>$edisplay_name</td>";
    print     "<td><table width='100%'><tr>";
    print      "<td width='30%'><font color='$ie_colour'><b>";
    print      $text{'colour_' . $ie_colour} ? $text{'colour_' . $ie_colour} : "<pre>#$ie_colour </pre>";
    print      "</b></font></pre></td>";
    print      "<td width='30%'><font color='$oe_colour'><b>";
    print      $text{'colour_' . $oe_colour} ? $text{'colour_' . $oe_colour} : "<pre>#$oe_colour </pre>";
    print      "</b></font></pre></td>";
    print      "<td width='30%'><font color='$coll_colour'><b>";
    print      $text{'colour_' . $coll_colour} ? $text{'colour_' . $coll_colour} : "<pre>#$coll_colour </pre>";
    print      "</b></font></pre></td>";
    print     "</tr></table></td><td align='right'>";
    if ($access{'configure'}) {
      if (!$edisplay_active) {
	print "<A href='edit_module.cgi?idx=$in{'idx'}&opt=display2&idx_mod=$idx_mod'>";
	print "<IMG src='images/nodata.png' border='0'>";
	print "</a>";
      } else {
	print "<A href='edit_module.cgi?idx=$in{'idx'}&opt=display2&idx_mod=$idx_mod'>";
	print "<IMG src='images/showdata.png' border='0'>";
	print "</a>";
      }
      print "<IMG src='images/space.png' border='0'>\n";
      print "<IMG src='images/space.png' border='0'>\n";
      print "<IMG src='images/space.png' border='0'>\n";
    }
    print "</td></tr>";

    $idx_mod++;
  }
  print "</table></td></tr>";

  local ($interface,
	 $idisplay_mode, $idisplay_active, $idisplay_name,
	 $i_colour, $o_colour,
	 $edisplay_mode, $edisplay_active, $edisplay_name,
	 $ie_colour, $oe_colour, $coll_colour,
	 $runstop
	);

  if ($in{'opt'} eq 'edit') {
    ($interface,
     $idisplay_mode, $idisplay_active, $idisplay_name,
     $i_colour, $o_colour,
     $edisplay_mode, $edisplay_active, $edisplay_name,
     $ie_colour, $oe_colour, $coll_colour,
     $runstop
    )
      = split(/,/, $my_config->{'net' . $in{'idx_mod'}});
  } else {
    $interface = '';
    $idisplay_mode = '';
    $idisplay_active = '';
    $idisplay_name = '';
    $i_colour = '';
    $o_colour = '';
    $edisplay_mode = '';
    $edisplay_active = '';
    $edisplay_name = '';
    $ie_colour = '';
    $oe_colour = '';
    $coll_colour = '';
    $runstop = '';
  }

  @if_list = split ( /\n/, `grep ":" /proc/net/dev | awk -F ":" '{print \$1 ":"}' | awk '{ print \$1}'`);

  print     "<tr>\n";
  print      "<td>";
  print       "<table border width='100%'>\n";
  print        "<tr $tb><td><b>";
  if ($in{'opt'} eq 'edit') {
    print $text{'module_subtitle_update'};
  } else {
    print $text{'word_addnew'};
  }
  print        "</b></td></tr>\n";
  print        "<tr $cb>";
  print         "<td>";
  print          "<form action='save_config.cgi'>";
  # start table for new interface montor
  print           "<table width='100%'>\n";
  print            "<tr><td>";
  # start table for interface select and names
  print             "<table width='100%'>";
  print              "<tr>";
  print               "<td><b>" . $text{'module_word_network'} . "</b></td>\n";
  if ($in{'opt'} eq 'edit') {
    print               "<td width='120'><tt>" . $interface . ":</tt></td>";
  } else {
    print               "<td width='120'><b>";
    print                 "<select name='net_def'>\n";
    my $i = 0;
    printf                 "<option value='" . $i++ . "' selected>%s\n", $text{'word_select'};
    foreach $iflist (@if_list) {
      printf               "<option value='" . $i++ . "'>%s\n", $iflist;
    }
    print                 "</select>";
    print               "</b></td>";
  }
  print               "<td><b>" . $text{'module_word_iname'} . "</b></td>\n";
  print               "<td>";
  printf "<input type='text' name='sname_def' size='20' maxlength='20' value='%s'>", $idisplay_name;
  print               "</td>";
  print               "<td><b>" . $text{'module_word_display'} . "</b></td>\n";
  print               "<td>";
  printf "<input type='checkbox' name='display_smode_def' value='normal' %s>",
    $idisplay_active eq 1 ? 'checked' : '';
  print               "</td>";
  print              "</tr>";
  print              "<tr>";
  print               "<td></td><td></td>";
  print               "<td><b>" . $text{'module_word_ename'} . "</b></td>\n";
  print               "<td>";
  printf "<input type='text' name='ename_def' size=20 maxlength=20 value='%s'>", $edisplay_name;
  print               "</td>";
  print               "<td><b>" . $text{'module_word_display'} . "</b></td>\n";
  print               "<td>";
  printf "<input type='checkbox' name='display_emode_def' value='error' %s>",
    $edisplay_active eq 1 ? 'checked' : '';
  print               "</td>";
  print              "</tr>";
  print             "</table>";
  # end table for interface select
  print            "</td></tr>";
  print            "<tr><td><hr>";
  # start table for colour select
  print             "<table width='100%'>";
  print              "<tr>";
  print               "<td><b>" . $text{'module_word_setcolours'} . "</b></td>";
  print              "</tr>";
  print              "<tr>";
  print               "<td><table width='100%'><tr>";
  print                "<td width='10'></td>";
  print                "<td><b>" . $text{'module_word_incomming'} . "</b></td>";
  print                "<td>";
  &option_colourselect('i_colour', $i_colour, $defaults->{'i_colour'});
  print                "</td>";
  print                "<td><b>" . $text{'module_word_ierrors'} . "</b></td>";
  print                "<td>";
  &option_colourselect('ie_colour', $o_colour, $defaults->{'o_colour'});
  print                "</td>";
  print               "</tr><tr>";
  print                "<td width='10'></td>";
  print                "<td><b>" . $text{'module_word_outgoing'} . "</b></td>";
  print                "<td>";
  &option_colourselect('o_colour', $ie_colour, $defaults->{'ie_colour'});
  print                "</td>";
  print                "<td><b>" . $text{'module_word_oerrors'} . "</b></td>";
  print                "<td>";
  &option_colourselect('oe_colour', $oe_colour, $defaults->{'oe_colour'});
  print                "</td>";
  print               "</tr><tr>";
  print                "<td width='10'></td>";
  print                "<td></td>";
  print                "<td></td>";
  print                "<td><b>" . $text{'module_word_coll'} . "</b></td>";
  print                "<td>";
  &option_colourselect('coll_colour', $coll_colour, $defaults->{'coll_colour'});
  print                "</td>";
  print               "</tr></table>";
  print              "</tr>";
  print             "</table>";
  print            "</td></tr>";
  print           "</table>";
  # end table for colour select
  print           "<input type='hidden' name='idx' value='" . $in{'idx'} . "'>";
  if ($in{'opt'} eq 'edit') {
    print           "<input type='hidden' name='opt' value='update_net'>";
    print           "<input type='hidden' name='net' value='" . $in{'idx_mod'} . "'>";
    print           "<input type='submit' value='". $text{'word_update'} . ":'>";
  } else {
    print           "<input type='hidden' name='opt' value='new_net'>";
    print           "<input type='hidden' name='net' value='" . $idx_mod . "'>";
    print           "<input type='submit' value='". $text{'word_addnew'} . ":'>";
  }
  print          "</form>";
  print         "</td>";
  print        "</tr>";
  print       "</table>";
  print      "</td>";
  print     "</tr>\n";
  print    "</table>";
  print   "</td>";
  print  "</tr>";
  print "</table>\n";
}

sub mod_config_save {
  local ($minfo, $my_config, $defaults) = ($_[0], $_[1], $_[2]);

  if ($in{'opt'} eq 'new_net') {
    if ($in{'net_def'} eq '0') {
      &error($text{'save_einvaled_selection'});
    }

    @if_list = split ( /\n/, `grep ":" /proc/net/dev | awk -F ":" '{print \$1}' | awk '{print \$1}'`);

    if (!-r $minfo->{'dbtables'} . @if_list[$in{'net_def'} - 1] . ".rrd") {
      local ($cmd);

      $cmd = $config{'rrdtool'};
      $cmd .= " create " . $minfo->{'dbtables'} . @if_list[$in{'net_def'} - 1] . ".rrd";
      $cmd .= " --start 0 --step 60";
      $cmd .= " DS:input:COUNTER:100:U:U";
      $cmd .= " DS:output:COUNTER:100:U:U";
      $cmd .= " DS:ierror:COUNTER:100:U:U";
      $cmd .= " DS:oerror:COUNTER:100:U:U";
      $cmd .= " DS:collision:COUNTER:100:U:U";
      $cmd .= " RRA:MAX:0.5:1:70";
      $cmd .= " RRA:MAX:0.5:2:750";
      $cmd .= " RRA:MAX:0.5:15:650";
      $cmd .= " RRA:MAX:0.5:50:650";

      `$cmd`;
    }

    $if_sname = $in{'sname_def'} ? $in{'sname_def'} : @if_list[$in{'net_def'} - 1] . ":";
    $if_ename = $in{'ename_def'} ? $in{'ename_def'} : (@if_list[$in{'net_def'} - 1] . ': Errors');

    $my_config->{'net' . $in{'net'}} =
      join(',',  @if_list[$in{'net_def'} - 1],
	   'normal', $in{'display_smode_def'} ? '1' : '0', $if_sname,
	   ($in{'i_colour1_def'} eq 'default') ? $defaults->{'i_colour'}
	   : $in{'i_colour2_def'} ? $in{'i_colour2_def'} : $in{'i_colour1_def'},
	   ($in{'o_colour1_def'} eq 'default') ? $defaults->{'o_colour'}
	   : $in{'o_colour2_def'} ? $in{'o_colour2_def'} : $in{'o_colour1_def'},
	   'error', $in{'display_emode_def'} ? '1' : '0',  $if_ename,
	   ($in{'ie_colour1_def'} eq 'default') ? $defaults->{'ie_colour'}
	   : $in{'ie_colour2_def'} ? $in{'ie_colour2_def'} : $in{'ie_colour1_def'},
	   ($in{'oe_colour1_def'} eq 'default') ? $defaults->{'oe_colour'}
	   : $in{'oe_colour2_def'} ? $in{'oe_colour2_def'} : $in{'oe_colour1_def'},
	   ($in{'coll_colour1_def'} eq 'default') ? $defaults->{'coll_colour'}
	   : $in{'coll_colour2_def'} ? $in{'coll_colour2_def'} : $in{'coll_colour1_def'},
	   0
	  );

    &write_config($minfo, $my_config, $defaults);
    &redirect("edit_module.cgi?idx=$in{'idx'}");
  } elsif ($in{'opt'} eq 'del_net') {
    my $idx_mod = $in{'net'};

    while ($my_config->{'net' . $idx_mod} ne '') {
      $my_config->{'net' . $idx_mod} = $my_config->{'net' . ($idx_mod + 1)};
      $idx_mod++;
    }
    $my_config->{'net' . ($idx_mod - 1)} = '';

    &write_config($minfo, $my_config, $defaults);

    unlink $minfo->{'dbatbles'} . "/net" . $in{'net'} . ".rrd";

    &redirect("edit_module.cgi?idx=$in{'idx'}");
  } elsif ($in{'opt'} eq 'update_net') {
    my $net = $in{'net'};

    local ($interface,
	   $idisplay_mode, $idisplay_active, $idisplay_name,
	   $i_colour, $o_colour,
	   $edisplay_mode, $edisplay_active, $edisplay_name,
	   $ie_colour, $oe_colour, $coll_colour,
	   $runstop
	  )
      = split(',', $my_config->{'net' . $in{'net'}});

    $if_sname = $in{'sname_def'} ? $in{'sname_def'} : $interface;
    $if_ename = $in{'ename_def'} ? $in{'ename_def'} : $interface . ' Errors';

    $my_config->{'net' . $in{'net'}} =
      join(',', $interface,
	   'normal', $in{'display_smode_def'} ? '1' : '0', $if_sname,
	   ($in{'i_colour1_def'} eq 'default') ? $defaults->{'i_colour'}
	   : $in{'i_colour2_def'} ? $in{'i_colour2_def'} : $in{'i_colour1_def'},
	   ($in{'o_colour1_def'} eq 'default') ? $defaults->{'o_colour'}
	   : $in{'o_colour2_def'} ? $in{'o_colour2_def'} : $in{'o_colour1_def'},
	   'error', $in{'display_emode_def'} ? '1' : '0',  $if_ename,
	   ($in{'ie_colour1_def'} eq 'default') ? $defaults->{'ie_colour'}
	   : $in{'ie_colour2_def'} ? $in{'ie_colour2_def'} : $in{'ie_colour1_def'},
	   ($in{'oe_colour1_def'} eq 'default') ? $defaults->{'oe_colour'}
	   : $in{'oe_colour2_def'} ? $in{'oe_colour2_def'} : $in{'oe_colour1_def'},
	   ($in{'coll_colour1_def'} eq 'default') ? $defaults->{'coll_colour'}
	   : $in{'coll_colour2_def'} ? $in{'coll_colour2_def'} : $in{'coll_colour1_def'},
	   $runstop
	  );

    &write_config($minfo, $my_config, $defaults);
    &redirect("edit_module.cgi?idx=$in{'idx'}");
  }

  &error( $in );
}

sub write_config {
  local ($minfo, $my_config, $defaults) = ($_[0], $_[1], $_[2]);

  local (%new_config, %sh_config);

  $new_config{'sample_rate'} = $my_config->{'sample_rate'};
  $sh_config{'sample_rate'} = $my_config->{'sample_rate'};
  $new_config{'runstop'} = $my_config->{'runstop'};

  $new_config{'db_uid'} = $my_config->{'db_uid'};

  my $i = 1;
  while ($my_config->{'net' . $i} ne '') {
    $new_config{'net' . $i} = $my_config->{'net' . $i};
    $i++;
  }

  my $idxnum = 1;
  my $idx_mod = 1;
  while ($new_config{'net' . $idx_mod} ne '') {
    local ($interface,
	   $idisplay_mode, $idisplay_active, $idisplay_name,
	   $i_colour, $o_colour,
	   $edisplay_mode, $edisplay_active, $edisplay_name,
	   $ie_colour, $oe_colour, $coll_colour
	  )
      = split(/,/, $new_config{'net' . $idx_mod});

    if ($idisplay_active) {
      $new_config{'display' .  $idxnum} = $idx_mod . ',1,' . $idxnum;
      $new_config{'module_index_title' .  $idxnum++} = $idisplay_name;
    }

    if ($edisplay_active) {
      $new_config{'display' .  $idxnum} = $idx_mod . ',2,' . $idxnum;
      $new_config{'module_index_title' .  $idxnum++} = $edisplay_name;
    }

    $sh_config{'net' . $idx_mod++} = $interface if (!$runstop);
  }

  unlink ($minfo->{'config_file'});
  unlink ($minfo->{'config_dir'} . '/network.cfg');

  &write_file($minfo->{'config_file'}, \%new_config);
  &write_file($minfo->{'config_dir'} . '/network.cfg', \%sh_config);
}

return 1;

